package cn.estsh.i3plus.ext.mes.pcn.apiservice.controller.busi;


import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesSortShippingCheckService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesRedisLockUtil;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesSortShippingCheckModel;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagement;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

@RestController
@Api(tags = "排序发运校验")
@RequestMapping(MesCommonConstant.MES_YANFEN + "/sort-shipping-check")
@Slf4j
public class MesSortShippingCheckController {

    @Autowired
    private IMesSortShippingCheckService sortShippingCheckService;

    @Autowired
    private MesRedisLockUtil redisLockUtil;

    private static final String key = ":BUSI_LOCK:SORT_SHIPPING_CHECK:";

    @GetMapping("/query-loading-order")
    @ApiOperation(value = "查询发运单")
    public ResultBean queryLoadingOrderNo(MesShippingOrderManagement shippingOrderManagement) {

        Boolean lockFlg = false;
        String moduleKey = null;
        String organizeCode = null;
        String userInfo = null;
        try {

            if (StringUtils.isEmpty(shippingOrderManagement.getShippingCode())) MesPcnException.throwBusiException("请扫描发运单!");

            organizeCode = !StringUtils.isEmpty(shippingOrderManagement.getOrganizeCode()) ? shippingOrderManagement.getOrganizeCode() : AuthUtil.getOrganize().getOrganizeCode();

            userInfo = !StringUtils.isEmpty(shippingOrderManagement.getUserInfo()) ? shippingOrderManagement.getUserInfo() : AuthUtil.getSessionUser().getUserName();

            moduleKey = organizeCode + key + "SHIPPING_CODE:" + shippingOrderManagement.getShippingCode();

            log.info("工厂:{} 排序发运扫描发运单 --- {} --- START --- 操作人:{} --- THREAD:{}", organizeCode, moduleKey, userInfo, Thread.currentThread().getName());

            //加锁
            lockFlg = redisLockUtil.redisLock(moduleKey);

            log.info("工厂:{} 排序发运扫描发运单 --- {} --- LOCKED --- 操作人:{} --- THREAD:{}", organizeCode, moduleKey, userInfo, Thread.currentThread().getName());

            // 数据校验
            ValidatorBean.checkNotNull(shippingOrderManagement.getShippingCode(), "发运单不能为空");
            shippingOrderManagement.setOrganizeCode(organizeCode);
            ConvertBean.serviceModelUpdate(shippingOrderManagement, userInfo);
            MesSortShippingCheckModel model = sortShippingCheckService.doShippingOrderNoQuery(shippingOrderManagement);
            return ResultBean.success("查询成功").setResultObject(model);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            //解锁
            if (lockFlg) {
                redisLockUtil.redisUnLock(lockFlg, moduleKey);
                log.info("工厂:{} 排序发运扫描发运单 --- {} --- UNLOCK --- 操作人:{} --- THREAD:{}", organizeCode, moduleKey, userInfo, Thread.currentThread().getName());
            }
        }
    }

    @PostMapping("/scan-sn")
    @ApiOperation(value = "扫描条码")
    public ResultBean saveSn(@RequestBody MesSortShippingCheckModel model) {

        Boolean lockFlg = false;
        String moduleKey = null;
        String organizeCode = null;
        String userInfo = null;
        try {

            if (StringUtils.isEmpty(model.getSn())) MesPcnException.throwBusiException("请扫描条码!");

            organizeCode = !StringUtils.isEmpty(model.getOrganizeCode()) ? model.getOrganizeCode() : AuthUtil.getOrganize().getOrganizeCode();

            userInfo = !StringUtils.isEmpty(model.getUserInfo()) ? model.getUserInfo() : AuthUtil.getSessionUser().getUserName();

            moduleKey = organizeCode + key + "SN:" + model.getSn();

            log.info("工厂:{} 排序发运扫描条码 --- {} --- START --- 操作人:{} --- THREAD:{}", organizeCode, moduleKey, userInfo, Thread.currentThread().getName());

            //加锁
            lockFlg = redisLockUtil.redisLock(moduleKey);

            log.info("工厂:{} 排序发运扫描条码 --- {} --- LOCKED --- 操作人:{} --- THREAD:{}", organizeCode, moduleKey, userInfo, Thread.currentThread().getName());

            // 数据校验
            ValidatorBean.checkNotNull(model.getShippingCode(), "发运单不能为空");
            ValidatorBean.checkNotNull(model.getDetailList(), "发运明细不能为空");
            ValidatorBean.checkNotNull(model.getSn(), "条码不能为空");

            sortShippingCheckService.saveSn(model, organizeCode, userInfo);
            return ResultBean.success("查询成功").setResultObject(model);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            //解锁
            if (lockFlg) {
                redisLockUtil.redisUnLock(lockFlg, moduleKey);
                log.info("工厂:{} 排序发运扫描条码 --- {} --- UNLOCK --- 操作人:{} --- THREAD:{}", organizeCode, moduleKey, userInfo, Thread.currentThread().getName());
            }
        }
    }

    @PostMapping("/scan-location-code")
    @ApiOperation(value = "扫描位置码")
    public ResultBean saveLocationCode(@RequestBody MesSortShippingCheckModel model) {

        Boolean lockFlg = false;
        String moduleKey = null;
        String organizeCode = null;
        String userInfo = null;
        try {

            if (StringUtils.isEmpty(model.getSn())) MesPcnException.throwBusiException("请扫描位置码!");

            organizeCode = !StringUtils.isEmpty(model.getOrganizeCode()) ? model.getOrganizeCode() : AuthUtil.getOrganize().getOrganizeCode();

            userInfo = !StringUtils.isEmpty(model.getUserInfo()) ? model.getUserInfo() : AuthUtil.getSessionUser().getUserName();

            moduleKey = organizeCode + key + "LOCATION_CODE:" + model.getSn();

            log.info("工厂:{} 排序发运扫描位置码 --- {} --- START --- 操作人:{} --- THREAD:{}", organizeCode, moduleKey, userInfo, Thread.currentThread().getName());

            //加锁
            lockFlg = redisLockUtil.redisLock(moduleKey);

            log.info("工厂:{} 排序发运扫描位置码 --- {} --- LOCKED --- 操作人:{} --- THREAD:{}", organizeCode, moduleKey, userInfo, Thread.currentThread().getName());

            // 数据校验
            ValidatorBean.checkNotNull(model.getShippingCode(), "发运单不能为空");
            ValidatorBean.checkNotNull(model.getDetailList(), "发运明细不能为空");
            ValidatorBean.checkNotNull(model.getSn(), "条码不能为空");
            ValidatorBean.checkNotNull(model.getLocationCode(), "位置码不能为空");

            sortShippingCheckService.saveSnAndLocationCode(model, organizeCode, userInfo);
            return ResultBean.success("查询成功").setResultObject(model);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            //解锁
            if (lockFlg) {
                redisLockUtil.redisUnLock(lockFlg, moduleKey);
                log.info("工厂:{} 排序发运扫描位置码 --- {} --- UNLOCK --- 操作人:{} --- THREAD:{}", organizeCode, moduleKey, userInfo, Thread.currentThread().getName());
            }
        }
    }


    @GetMapping("/query-auto-push-order")
    @ApiOperation(value = "自动推单")
    public ResultBean queryAutoPushOrder(String shippingGroupNo, String organizeCode) {

        try {
            organizeCode = !StringUtils.isEmpty(organizeCode) ? organizeCode : AuthUtil.getOrganize().getOrganizeCode();
            MesShippingOrderManagement order = sortShippingCheckService.queryShippingOrderNoByAutoPushOrder(shippingGroupNo, organizeCode);
            return ResultBean.success("查询成功").setResultObject(order);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
