package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesAssemblyExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdRuleContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblyNosortContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblySortContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class MesAssemblyExtService implements IMesAssemblyExtService {

    @Autowired
    private MesWorkOrderAssemblyRepository workOrderAssemblyRepository;

    @Autowired
    private MesProductionAssemblyRepository productionAssemblyRepository;

    @Autowired
    private MesProductionAssemblyUniqueRepository productionAssemblyUniqueRepository;

    @Autowired
    private MesAssemblyNosortCfgRepository assemblyNosortCfgRepository;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    @Autowired
    private MesFileRepository fileRepository;

    //【排序线】获取生产工单装配件绑定记录
    @Override
    public List<MesProductionAssemblySortContext> getProductionAssemblySortContextList(MesProdRuleContext prodRuleContext) {

        if (null == prodRuleContext) return null;

        if (StringUtils.isEmpty(prodRuleContext.getOrganizeCode()) || StringUtils.isEmpty(prodRuleContext.getWorkCenterCode()) ||
                StringUtils.isEmpty(prodRuleContext.getWorkCellCode()) || StringUtils.isEmpty(prodRuleContext.getProcessCode()) || StringUtils.isEmpty(prodRuleContext.getProductSn())) return null;

        //【排序线】获取生产工单装配件清单
        List<MesWorkOrderAssembly> workOrderAssemblyList = getWorkOrderAssemblyList(prodRuleContext);

        if (CollectionUtils.isEmpty(workOrderAssemblyList)) return null;

        //获取排序FILE文件URL
        Map<Long, String> fileMap = getFileMap(prodRuleContext.getOrganizeCode(), filterSortFileIdList(workOrderAssemblyList));

        List<MesProductionAssemblySortContext> productionAssemblySortContextList = new ArrayList<>();

        workOrderAssemblyList.forEach(o -> productionAssemblySortContextList.add(new MesProductionAssemblySortContext().copy(o).fileUrl(fileMap)));

        return productionAssemblySortContextList;
    }

    //【排序线】获取生产工单装配件清单
    @Override
    public List<MesWorkOrderAssembly> getWorkOrderAssemblyList(MesProdRuleContext prodRuleContext) {

        if (null == prodRuleContext) return null;

        if (StringUtils.isEmpty(prodRuleContext.getOrganizeCode()) || StringUtils.isEmpty(prodRuleContext.getWorkCenterCode()) ||
                StringUtils.isEmpty(prodRuleContext.getWorkCellCode()) || StringUtils.isEmpty(prodRuleContext.getProcessCode()) || StringUtils.isEmpty(prodRuleContext.getProductSn())) return null;

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(prodRuleContext.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(prodRuleContext.getWorkCenterCode(), MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(prodRuleContext.getWorkCellCode(), MesPcnExtConstWords.WORK_CELL_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(prodRuleContext.getProcessCode(), MesPcnExtConstWords.PROCESS_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(prodRuleContext.getProductSn(), MesPcnExtConstWords.PRODUCT_SN, packBean);

        return workOrderAssemblyRepository.findByHqlWhere(packBean);

    }

    //获取装配件绑定记录
    @Override
    public List<MesProductionAssembly> getProductionAssemblyList(String organizeCode, Integer dataSource, List<Long> sourceIdList) {

        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(dataSource) || CollectionUtils.isEmpty(sourceIdList)) return null;

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(dataSource, MesPcnExtConstWords.DATD_SOURCE, packBean);
        if (sourceIdList.size() == 1) DdlPreparedPack.getNumEqualPack(sourceIdList.get(0), MesPcnExtConstWords.SOURCE_ID, packBean);
        else DdlPreparedPack.getInPackList(sourceIdList, MesPcnExtConstWords.SOURCE_ID, packBean);

        return productionAssemblyRepository.findByHqlWhere(packBean);

    }

    @Override
    public List<MesProductionAssembly> getProductionAssemblyList(String organizeCode, String assemblySn) {
        return productionAssemblyRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.ASSEMBLY_SN},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), assemblySn});
    }

    @Override
    public List<MesProductionAssemblyUnique> getProductionAssemblyUniqueList(String organizeCode, String assemblySn) {
        return productionAssemblyUniqueRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.ASSEMBLY_SN},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), assemblySn});
    }

    //获取非排序装配件配置信息
    @Override
    public List<MesProductionAssemblyNosortContext> getProductionAssemblyNosortContextList(MesProdRuleContext prodRuleContext) {

        if (null == prodRuleContext || StringUtils.isEmpty(prodRuleContext.getOrganizeCode()) || StringUtils.isEmpty(prodRuleContext.getPid())) return null;

        //获取非排序装配件配置信息
        List<MesAssemblyNosortCfg> assemblyNosortCfgList = getAssemblyNosortCfgList(prodRuleContext.getOrganizeCode(), prodRuleContext.getPid());

        if (CollectionUtils.isEmpty(assemblyNosortCfgList)) return null;

        //获取非排序FILE文件URL
        Map<Long, String> fileMap = getFileMap(prodRuleContext.getOrganizeCode(), filterNosortFileIdList(assemblyNosortCfgList));

        List<MesProductionAssemblyNosortContext> productionAssemblyNosortContextList = new ArrayList<>();

        //遍历装配件规则, 实例化装配件清单对象, 判断是否配置可复用
        assemblyNosortCfgList.forEach(o -> productionAssemblyNosortContextList.add(
                new MesProductionAssemblyNosortContext().copy(prodRuleContext, o, getRepeatAssemblySn(prodRuleContext.getOrganizeCode(), prodRuleContext.getWorkCenterCode(), prodRuleContext.getWorkCellCode(), o)).fileUrl(fileMap))
        );

        return productionAssemblyNosortContextList;

    }

    //获取非排序装配件配置信息
    @Override
    public List<MesAssemblyNosortCfg> getAssemblyNosortCfgList(String organizeCode, Long pid) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(pid)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(pid, MesPcnExtConstWords.PID, packBean);
        return assemblyNosortCfgRepository.findByHqlWhere(packBean);
    }

    @Override
    public List<MesAssemblyNosortCfg> getAssemblyNosortCfgList(String organizeCode, String equipmentCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipmentCode)) return null;
        return assemblyNosortCfgRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.EQUIPMENT_CODE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), equipmentCode});
    }

    @Override
    public List<MesProductionAssemblyNosortContext> getProductionAssemblyNosortContextList(String organizeCode, String equipmentCode) {

        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(equipmentCode)) return null;

        List<MesAssemblyNosortCfg> assemblyNosortCfgList = getAssemblyNosortCfgList(organizeCode, equipmentCode);
        if (CollectionUtils.isEmpty(assemblyNosortCfgList)) return null;

        List<MesProductionAssemblyNosortContext> productionAssemblyNosortContextList = new ArrayList<>();

        assemblyNosortCfgList.forEach(o -> productionAssemblyNosortContextList.add(new MesProductionAssemblyNosortContext().copy(o)));

        return productionAssemblyNosortContextList;
    }

    @Override
    public List<MesWorkOrderAssembly> getWorkOrderAssemblyList(String organizeCode, String workOrderNo, String productSn, String craftCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(workOrderNo) || StringUtils.isEmpty(productSn) || StringUtils.isEmpty(craftCode)) return null;
        return workOrderAssemblyRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.WORK_ORDER_NO, MesPcnExtConstWords.PRODUCT_SN, MesPcnExtConstWords.CRAFT_CODE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), workOrderNo, productSn, craftCode});
    }

    @Override
    public List<MesWorkOrderAssembly> getWorkOrderAssemblyList(String organizeCode, String workCenterCode, String workCellCode, String workOrderNo, List<String> productSnList) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(workCenterCode) || StringUtils.isEmpty(workCellCode) || StringUtils.isEmpty(workOrderNo) || CollectionUtils.isEmpty(productSnList)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(workCellCode, MesPcnExtConstWords.WORK_CELL_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(workOrderNo, MesPcnExtConstWords.WORK_ORDER_NO, packBean);
        if (productSnList.size() == 1) DdlPreparedPack.getStringEqualPack(productSnList.get(0), MesPcnExtConstWords.PRODUCT_SN, packBean);
        else DdlPreparedPack.getInPackList(productSnList, MesPcnExtConstWords.PRODUCT_SN, packBean);
        return workOrderAssemblyRepository.findByHqlWhere(packBean);
    }

    //验证装配件规则对应的ID是否存在可复用装配件条码
    private String getRepeatAssemblySn(String organizeCode, String workCenterCode, String workCellCode, MesAssemblyNosortCfg assemblyNosortCfg) {

        if (null == assemblyNosortCfg || StringUtils.isEmpty(assemblyNosortCfg.getIsRepeat()) || assemblyNosortCfg.getIsRepeat().compareTo(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()) == 0) return null;

        return productionCustomContextStepService.getRepeatAssemblySn(organizeCode, workCenterCode, workCellCode, assemblyNosortCfg.getId().toString());

    }

    //搜集非排序文件ID集合
    private List<Long> filterNosortFileIdList(List<MesAssemblyNosortCfg> assemblyNosortCfgList) {
        if (CollectionUtils.isEmpty(assemblyNosortCfgList)) return null;
        List<Long> imageFileIdList = assemblyNosortCfgList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getImageFileId()))).map(MesAssemblyNosortCfg::getImageFileId).collect(Collectors.toList());
        List<Long> audioFileIdList = assemblyNosortCfgList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getAudioFileId()))).map(MesAssemblyNosortCfg::getAudioFileId).collect(Collectors.toList());
        return filterFileIdList(imageFileIdList, audioFileIdList);
    }

    //搜集排序文件ID集合
    private List<Long> filterSortFileIdList(List<MesWorkOrderAssembly> workOrderAssemblyList) {
        if (CollectionUtils.isEmpty(workOrderAssemblyList)) return null;
        List<Long> imageFileIdList = workOrderAssemblyList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getImageFileId()))).map(MesWorkOrderAssembly::getImageFileId).collect(Collectors.toList());
        List<Long> audioFileIdList = workOrderAssemblyList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getAudioFileId()))).map(MesWorkOrderAssembly::getAudioFileId).collect(Collectors.toList());
        return filterFileIdList(imageFileIdList, audioFileIdList);
    }

    //搜集文件ID集合
    private List<Long> filterFileIdList(List<Long> imageFileIdList, List<Long> audioFileIdList) {
        if (!CollectionUtils.isEmpty(imageFileIdList) && CollectionUtils.isEmpty(audioFileIdList)) return imageFileIdList;
        if (CollectionUtils.isEmpty(imageFileIdList) && !CollectionUtils.isEmpty(audioFileIdList)) return audioFileIdList;
        imageFileIdList.addAll(audioFileIdList);
        return imageFileIdList;
    }

    //获取FILE文件URL
    private Map<Long, String> getFileMap(String organizeCode, List<Long> fileIdList) {
        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(fileIdList)) return null;
        fileIdList = fileIdList.stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (fileIdList.size() == 1) DdlPreparedPack.getNumEqualPack(fileIdList.get(0), MesPcnExtConstWords.ID, packBean);
        DdlPreparedPack.getInPackList(fileIdList, MesPcnExtConstWords.ID, packBean);
        List<MesFile> fileDbList = fileRepository.findByHqlWhere(packBean);
        return CollectionUtils.isEmpty(fileIdList) ? null : fileDbList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getFileUrl()))).collect(Collectors.toMap(MesFile::getId, MesFile::getFileUrl));
    }

}
