package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesCustomerSnTransformService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkOrderExtService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.aspect.MonitorLog;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesQueueOrder;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkOrderRepository;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class MesWorkOrderExtService implements IMesWorkOrderExtService {

    @Autowired
    private MesWorkOrderRepository workOrderRepository;

    @Autowired
    private IMesProduceSnExtService produceSnExtService;

    @Autowired
    private IMesCustomerSnTransformService customerSnTransformService;

    //根据生产工单号集合 查询生产工单数据
    @Override
    public List<MesWorkOrder> getWorkOrderList(String organizeCode, List<String> workOrderNoList) {

        if (StringUtils.isEmpty(organizeCode) || CollectionUtils.isEmpty(workOrderNoList)) return null;

        workOrderNoList = workOrderNoList.size() == 1 ? workOrderNoList : workOrderNoList.stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (workOrderNoList.size() == 1) DdlPreparedPack.getStringEqualPack(workOrderNoList.get(0), MesPcnExtConstWords.WORK_ORDER_NO, packBean);
        else DdlPreparedPack.getInPackList(workOrderNoList, MesPcnExtConstWords.WORK_ORDER_NO, packBean);

        return workOrderRepository.findByHqlWhere(packBean);

    }

    //根据生产工单ID查询生产工单信息
    @Override
    public MesWorkOrder getWorkOrder(String organizeCode, Long id) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(id)) return null;
        return workOrderRepository.getByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.ID},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), id});
    }

    @Override
    public MesWorkOrder getWorkOrder(String organizeCode, String workOrderNo) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(workOrderNo)) return null;
        return workOrderRepository.getByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.WORK_ORDER_NO},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), workOrderNo});
    }

    //根据生产工单顺序号,生产线代码查询上一个生产工单信息
    @Override
    public MesWorkOrder getWorkOrder(String organizeCode, String workCenterCode, Long produceSeq) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(workCenterCode) || (StringUtils.isEmpty(produceSeq) || produceSeq.compareTo(0L) == 0)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getNumberSmallerPack(produceSeq, MesPcnExtConstWords.PRODUCE_SEQ, packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{MesPcnExtConstWords.PRODUCE_SEQ}, packBean);
        return workOrderRepository.getByProperty(packBean);
    }

    //根据生产工单顺序号,生产线代码,零件生产组代码查询上一个生产工单信息
    @Override
    public MesWorkOrder getWorkOrder(String organizeCode, String workCenterCode, Long produceSeq, String partProdGroupCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(workCenterCode) || StringUtils.isEmpty(partProdGroupCode) || StringUtils.isEmpty(produceSeq) || produceSeq.compareTo(0L) == 0) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(partProdGroupCode, MesPcnExtConstWords.PART_PROD_GROUP_CODE, packBean);
        DdlPreparedPack.getNumberSmallerPack(produceSeq, MesPcnExtConstWords.PRODUCE_SEQ, packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{MesPcnExtConstWords.PRODUCE_SEQ}, packBean);
        return workOrderRepository.getByProperty(packBean);
    }

    //根据生产线代码, 零件生产组代码查询生产工单信息【排序】
    @Override
    public List<MesWorkOrder> getWorkOrderList(String organizeCode, String workCenterCode, String partProdGroupCode) {
        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(workCenterCode) || StringUtils.isEmpty(partProdGroupCode)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(partProdGroupCode, MesPcnExtConstWords.PART_PROD_GROUP_CODE, packBean);
        return workOrderRepository.findByHqlWhere(packBean);
    }

    @Override
    @MonitorLog
    public MesWorkOrder insert(MesWorkOrder item) {
        item.setFid(UUID.randomUUID().toString());
        return workOrderRepository.insert(item);
    }

    @Override
    @MonitorLog
    public void update(MesWorkOrder item) {
        workOrderRepository.update(item);
    }

    @Override
    public List<MesWorkOrder> getWorkOrderListByShiftCode(String organizeCode, String workCenterCode, String shiftCode) {
        String currentTime = DateUtil.formatDateTime(new Date());
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(workCenterCode, MesPcnExtConstWords.WORK_CENTER_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(shiftCode, MesPcnExtConstWords.SHIFT_CODE, packBean);
        DdlPreparedPack.getStringSmallerNotEqualPack(currentTime, "planStartTime", packBean);
        DdlPreparedPack.getStringBiggerNotEqualPack(currentTime, "planEndTime", packBean);

        return workOrderRepository.findByHqlWhere(packBean);
    }

    @Override
    public Map<String, Map<String, Object>> getWorkOrderMapSort(String organizeCode, String workCenterCode, String workCellCode, List<String> snList) {

        if (StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(workCenterCode) || StringUtils.isEmpty(workCellCode) || CollectionUtils.isEmpty(snList)) return null;

        snList = snList.size() == 1 ? snList : snList.stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());

        Map<String, Map<String, Object>> workOrderMap = new HashMap<>();

        for (String sn : snList) {

            if (StringUtils.isEmpty(sn)) continue;

            Map<String, Object> itemMap = new HashMap<>();

            if (sn.length() == MesPcnExtConstWords.CUSTOMER_SN_LENGTH_GM) {
                MesProduceSn produceSn = produceSnExtService.getProduceSnByCustSn(organizeCode, customerSnTransformService.transformBarCodeGm(sn));
                if (null == produceSn || StringUtils.isEmpty(produceSn.getWorkOrderNo())) {
                    itemMap.put(MesPcnExtConstWords.MESSAGE, String.format("请检查工单及客户条码信息,客户条码[%s]信息无效!", sn));
                    continue;
                } else itemMap.put(MesProduceSn.class.getSimpleName(), produceSn);

                MesWorkOrder workOrder = getWorkOrder(organizeCode, produceSn.getWorkOrderNo());
                if (null == workOrder) {
                    itemMap.put(MesPcnExtConstWords.MESSAGE, String.format("请检查工单信息,客户条码[%s]对应的加工单[%s]信息不存在!", sn, produceSn.getWorkOrderNo()));
                    continue;
                } else itemMap.put(MesWorkOrder.class.getSimpleName(), workOrder);

                MesQueueOrder queueOrder = produceSnExtService.getQueueOrder(organizeCode, workCenterCode, workCellCode, produceSn.getProductSn());
                if (null == queueOrder) itemMap.put(MesPcnExtConstWords.MESSAGE, String.format("请检查工位队列信息,客户条码[%s]工位队列信息不存在!", sn));
                else itemMap.put(MesQueueOrder.class.getSimpleName(), queueOrder);

            } else {
                MesProduceSn produceSn = produceSnExtService.getProduceSn(organizeCode, sn, sn);
                if (null == produceSn) produceSn = produceSnExtService.getProduceSnByCustSn(organizeCode, sn);
                if (null == produceSn) {
                    itemMap.put(MesPcnExtConstWords.MESSAGE, String.format("请检查工单及产品条码信息,加工单[%s]不存在产品条码信息!", sn));
                    continue;
                } else itemMap.put(MesProduceSn.class.getSimpleName(), produceSn);

                MesWorkOrder workOrder = getWorkOrder(organizeCode, produceSn.getWorkOrderNo());
                if (null == workOrder) {
                    itemMap.put(MesPcnExtConstWords.MESSAGE, String.format("请检查工单信息,加工单[%s]信息无效!", produceSn.getWorkOrderNo()));
                    continue;
                } else itemMap.put(MesWorkOrder.class.getSimpleName(), workOrder);

                MesQueueOrder queueOrder = produceSnExtService.getQueueOrder(organizeCode, workCenterCode, workCellCode, produceSn.getProductSn());
                if (null == queueOrder) itemMap.put(MesPcnExtConstWords.MESSAGE, String.format("请检查工位队列信息,加工单[%s]工位队列信息不存在!", produceSn.getWorkOrderNo()));
                else itemMap.put(MesQueueOrder.class.getSimpleName(), queueOrder);

            }

            workOrderMap.put(sn, itemMap);

        }

        return workOrderMap;
    }

}
