package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.*;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPartContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesQueueOrder;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrderAssembly;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : 生产工单顺序防错工步 【排序通用汽车客户订单号】
 * @Author : wangjie
 **/
@Slf4j
@Service("mesProduceSeqCheckCustOrderNoStepService")
public class MesProduceSeqCheckCustOrderNoStepService extends MesProduceSeqCheckStepService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private IMesWorkOrderExtService workOrderExtService;

    @Autowired
    private IMesProduceSnExtService produceSnExtService;

    @Autowired
    private IMesAssemblyExtService assemblyExtService;

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        //获取上下文信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.dispatchCurCellEquipment(reqBean);

        //获取生产过程上下文对象有异常信息 抛出异常
        if (!productionProcessContext.getSuccess()) stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), productionProcessContext.getMessage());

        //存储生产过程上下文对象
        productionProcessContextStepService.dispatchProductionProcessContext(reqBean, productionProcessContext);

        //获取上下文产出零件信息
        List<MesProductionPartContext> productionPartContextList = productionDispatchContextStepService.getProductionPartContext(reqBean);

        //验证是否存在加工单
        if (!checkIsExistWorkOrder(productionPartContextList)) return stepSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "当前无加工单信息,无需进行加工单客户订单号顺序防错验证!");

        //验证工位是否设置需要顺序防错
        if (!isNeedCheckProduceSeq(productionPartContextList))
            return stepSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, String.format("生产线[%s]工位[%s]未设置顺序防错,无需进行加工单客户订单号顺序防错验证!", reqBean.getWorkCenterCode(), reqBean.getWorkCellCode()));

        //验证是否存在工艺强过码, 存在则保存 并返回强过的加工单
        List<String> workOrderNoList2Jump = doHandleCraftJumpCode2Custom(reqBean, productionPartContextList);
        if (!CollectionUtils.isEmpty(workOrderNoList2Jump))
            return stepSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog().scanInfo(workOrderNoList2Jump.toString()), stepResult, String.format("加工单%s跳过客户订单号顺序防错验证成功!", workOrderNoList2Jump.toString()));

        //验证是否已经全部通过
        if (!isExistNeedCheckProduceSeq2Custom(productionPartContextList)) return stepSuccessCompleteAndSendMsgReturn(reqBean, resultBean, stepResult, "加工单均已通过客户订单号顺序防错验证!");

        //顺序防错验证处理
        doHandleProduceSeqCheck(reqBean, resultBean, stepResult, productionProcessContext, productionPartContextList);

        return stepResult;

    }

    //验证是否存在工艺强过码, 存在则保存 并返回强过的加工单
    private List<String> doHandleCraftJumpCode2Custom(StationRequestBean reqBean, List<MesProductionPartContext> productionPartContextList) {

        List<String> workOrderNoList2Jump = null;

        for (MesProductionPartContext productionPartContext : productionPartContextList) {

            if (null == productionPartContext || productionPartContext.getCustomCheckSeqResult().compareTo(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()) != 0 || StringUtils.isEmpty(productionPartContext.getCraftJumpCode())) continue;

            if (CollectionUtils.isEmpty(workOrderNoList2Jump)) workOrderNoList2Jump = new ArrayList<>();

            workOrderNoList2Jump.add(productionPartContext.getWorkOrderNo());

            //验证成功不需要标记【checkSeqResult】
            productionPartContext.customCheckSeqResult(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());

        }

        //保存上下文产出零件信息
        if (!CollectionUtils.isEmpty(workOrderNoList2Jump)) productionDispatchContextStepService.dispatchProductionPartContext(reqBean, productionPartContextList);

        return workOrderNoList2Jump;

    }

    //验证是否已经全部通过
    private Boolean isExistNeedCheckProduceSeq2Custom(List<MesProductionPartContext> productionPartContextList) {
        Optional<MesProductionPartContext> optional = productionPartContextList.stream().filter(o -> (null != o &&
                o.getIsFinishCode().compareTo(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()) == 0 && !StringUtils.isEmpty(o.getCustOrderNo()) && o.getCustomCheckSeqResult().compareTo(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) != 0)).findFirst();
        return (null == optional || !optional.isPresent()) ? false : true;
    }

    //顺序防错验证处理
    private void doHandleProduceSeqCheck(StationRequestBean reqBean, StationResultBean resultBean, StepResult stepResult, MesProductionProcessContext productionProcessContext, List<MesProductionPartContext> productionPartContextList) {

        List<String> workOrderNoList = new ArrayList<>();

        for (MesProductionPartContext productionPartContext : productionPartContextList) {

            if (null == productionPartContext || StringUtils.isEmpty(productionPartContext.getWorkOrderNo()) || productionPartContext.getCheckSeqResult().compareTo(MesPcnExtConstWords.ZERO) != 0) continue;

            //顺序防错验证
            Boolean result = doProduceSeqCheckSort(reqBean, stepResult, productionProcessContext, productionPartContext);

            //验证失败需要同时标记【checkSeqResult】,但验证成功不需要标记【checkSeqResult】
            if (result) productionPartContext.customCheckSeqResult(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            else productionPartContext.customCheckSeqResult(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()).checkSeqResult(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());

            workOrderNoList.add(productionPartContext.getWorkOrderNo());

        }

        //保存上下文产出零件信息
        productionDispatchContextStepService.dispatchProductionPartContext(reqBean, productionPartContextList);

        if (stepResult.isCompleted()) this.sendMessage(reqBean, resultBean.writeDbLog().scanInfo(workOrderNoList.toString()), String.format("加工单%s客户订单号顺序防错验证成功!", workOrderNoList.toString()), MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
        else this.sendMessage(reqBean, resultBean.writeDbLog().scanInfo(workOrderNoList.toString()), stepResult.getMsg(), MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);

    }

    //前道工艺防错验证 【排序客户订单号】
    private Boolean doProduceSeqCheckSort(StationRequestBean reqBean, StepResult stepResult, MesProductionProcessContext productionProcessContext, MesProductionPartContext productionPartContext) {

        if (StringUtils.isEmpty(productionPartContext.getCustOrderNo()) || productionPartContext.getCustOrderNo().length() <= 3) return true;

        String message = StringUtils.isEmpty(stepResult.getMsg()) ? MesPcnExtConstWords.EMPTY : stepResult.getMsg() + MesPcnExtConstWords.SEMICOLON;

        //根据生产线代码,零件生产组代码查询产工单信息
        List<MesWorkOrder> workOrderList = workOrderExtService.getWorkOrderList(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode(), productionPartContext.getPartProdGroupCode());
        //1GA2752364 【1GA=客户工厂号;  2752364 = 客户订单号】 根据固定的前三位分组, 基于同一个零件生产组下相同前缀判断客户订单的流水号
        Map<String, List<MesWorkOrder>> custOrderNoMap = CollectionUtils.isEmpty(workOrderList) ? null : workOrderList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getCustOrderNo()) && o.getCustOrderNo().length() > 3)).collect(Collectors.groupingBy(o -> o.getCustOrderNo().substring(0, 3)));
        workOrderList = CollectionUtils.isEmpty(custOrderNoMap) ? null : custOrderNoMap.get(productionPartContext.getCustOrderNo().substring(0, 3));
        workOrderList = CollectionUtils.isEmpty(workOrderList) ? null : workOrderList.stream().filter(o -> null != o).sorted(Comparator.comparing(MesWorkOrder::getCustOrderNo).reversed()).collect(Collectors.toList());
        Optional<MesWorkOrder> workOrderOp = CollectionUtils.isEmpty(workOrderList) ? null : workOrderList.stream().filter(o -> (null != o && o.getCustOrderNo().compareTo(productionPartContext.getCustOrderNo()) <= 0)).findFirst();
        if (null == workOrderOp || !workOrderOp.isPresent()) return true;

        MesWorkOrder workOrder = workOrderOp.get();
        if (workOrder.getWorkOrderStatus().compareTo(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue()) < 0)
            return stepResult.isCompleted(false).msg(String.format("%s加工单[%s]客户订单号[%s]顺序防错验证失败,前一个加工单[%s]客户订单号[%s]工单状态[%s]",
                    message, productionPartContext.getWorkOrderNo(), productionPartContext.getCustOrderNo(), workOrder.getWorkOrderNo(), workOrder.getCustOrderNo(), MesExtEnumUtil.ORDER_STATUS.valueOfDescription(workOrder.getWorkOrderStatus()))).isCompleted();

        if (workOrder.getWorkOrderStatus().compareTo(MesExtEnumUtil.ORDER_STATUS.CLOSE.getValue()) >= 0) return true;

        //根据生产工单,生产线,工位查询工位队列信息
        List<MesQueueOrder> queueOrderList = produceSnExtService.getQueueOrderList(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode(), reqBean.getWorkCellCode(), workOrder.getWorkOrderNo());
        Optional<MesQueueOrder> optional = CollectionUtils.isEmpty(queueOrderList) ? null : queueOrderList.stream().filter(o -> (null != o && o.getStatus().compareTo(MesExtEnumUtil.QUEUE_ORDER_STATUS.CREATE.getValue()) == 0)).findFirst();
        if (null == optional || !optional.isPresent()) return true;

        //根据未完成零件条码获取生产工单装配件清单
        List<String> productSnList = queueOrderList.stream().filter(o -> (null != o && o.getStatus().compareTo(MesExtEnumUtil.QUEUE_ORDER_STATUS.CREATE.getValue()) == 0)).map(MesQueueOrder::getProductSn).collect(Collectors.toList());
        List<MesWorkOrderAssembly> workOrderAssemblyList = assemblyExtService.getWorkOrderAssemblyList(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode(), reqBean.getWorkCellCode(), workOrder.getWorkOrderNo(), productSnList);
        Map<String, List<MesWorkOrderAssembly>> productSnMap = CollectionUtils.isEmpty(workOrderAssemblyList) ? null : workOrderAssemblyList.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesWorkOrderAssembly::getProductSn));
        if (CollectionUtils.isEmpty(productSnMap)) return true;

        return stepResult.isCompleted(false).msg(String.format("%s加工单[%s]客户订单号[%s]顺序防错验证失败,前一个加工单[%s]客户订单号[%s]在当前工位存在未完成队列条码%s",
                message, productionPartContext.getWorkOrderNo(), productionPartContext.getCustOrderNo(), workOrder.getWorkOrderNo(), workOrder.getCustOrderNo(), productSnMap.keySet().toString())).isCompleted();

    }

}
